/***************************************************************************
 *   Copyright (C) 2009 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "autorotation.h"
#include <math.h>
//#include <QDebug>

AutoRotation::AutoRotation(QWidget *parent)
 : QWidget(parent)
{

	IconBasePath = "/usr/share/pixmaps/openmoko-panel-plugin-qt/";
	QIcon icon(IconBasePath + "autorotation.png");
	setWindowIcon(icon); 
	systrayicon = new QSystemTrayIcon(icon, this);

	connect(systrayicon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	systrayicon->show();



	VLayout = new QVBoxLayout();
	HLayout = new QHBoxLayout();

	InfoBox = new OMInfoBox(QObject::tr("g-force"), this);

	AxisList << QObject::tr("x-axis:") << QObject::tr("y-axis:") << QObject::tr("z-axis:") << QObject::tr("res:");


	int n;
	for ( n = 0; n < 4; n++) {
		InfoBox->addPanel(AxisList.at(n), "");
		
	}

	VLayout->addWidget(InfoBox);	

	OnOffString << QObject::tr("Turn Autorotation on") << QObject::tr("Turn Autorotation off");


	CheckButton = new OMCheckButton(OnOffString.at(0),this);
	CheckButton->setCheckState(false);
	connect(CheckButton,SIGNAL(stateChanged ( bool )), this, SLOT(CheckButtonClicked( bool )));
	HLayout->addWidget(CheckButton);

	ManualRotation = new OMManualRotation( this );
	HLayout->addWidget( ManualRotation );
	connect( ManualRotation, SIGNAL( manualRotationEvent( int ) ), this, SLOT( manualRotationSelected (int ) ) ); 

	VLayout->addLayout(HLayout);

	DialogButtons = new OMDialogButtons(OMFlags::QuitButton | OMFlags::CloseButton, this);
	VLayout->addWidget( DialogButtons );
	connect(DialogButtons, SIGNAL(clicked(QFlags< OMFlags :: Flag >)), this, SLOT(DialogButtonClicked(QFlags< OMFlags :: Flag >)));


	Settings = new QSettings("openmoko","openmoko-panel-plugin-autorotation",this);
	CheckButton->setCheckState(Settings->value("enabled").toBool());

	Timer = new QTimer(this);
	Timer->setInterval(100);
	connect(Timer, SIGNAL(timeout()), this, SLOT(readAccelerometer()));
	Timer->start();


	evFile = new QFile("/dev/input/event3");
	evFile->open(QIODevice::ReadOnly);
	evsize = sizeof( struct input_event );
	evdata = (char *)malloc(evsize);

	rot = 0; newrot = 0;
	isWaiting = false;
	angle = 0;
	oldangle = 0;

	XDisplay = XOpenDisplay(":0");
	XScreen = DefaultScreen(XDisplay);
	rootWindow = RootWindow(XDisplay, XScreen);

	setLayout(VLayout);

}


AutoRotation::~AutoRotation()
{
	
	evFile->close();
	free(evdata);
}




/*!
    \fn AutoRotation::updateView()
 */
void AutoRotation::readAccelerometer()
{
	evFile->read( evdata, evsize);
	memcpy(&ev, evdata, evsize);

	QString tmp;

	if (ev.type == 3) {
		absa[ev.code] = ev.value;
		tmp.setNum( absa[ev.code] / 100 );
		InfoBox->changePanelValue(AxisList.at(ev.code), tmp );

		absa[3] = sqrtf( powf(absa[0],2) +  powf(absa[1],2) + powf(absa[2],2) );
		tmp.setNum( absa[3] / 100 );
		InfoBox->changePanelValue(AxisList.at(3), tmp );
	
	}

		angle = acos( -1 * absa[1] / sqrtf( powf(absa[0],2) +  powf(absa[1],2) ));
 		angle = 180 * angle / M_PI;

		if (absa[0] < 0) angle = 360 - angle;
		rot = (int)round( angle / 90 ) % 4;

		if ((rot != newrot) && (!isWaiting) && (CheckButton->isChecked())) {
			QTimer::singleShot(1000, this, SLOT(rotateScreen()));
			isWaiting = true;
			newrot = rot;
			} 

	
}



void AutoRotation::rotateScreen()
{
	isWaiting = false;

	if (rot == newrot) {
	XRRRotations(XDisplay, XScreen, &XRotation);
	XConfig = XRRGetScreenInfo(XDisplay, rootWindow);
	current_Xsize = XRRConfigCurrentConfiguration(XConfig, &XRotation);

	

	XRotation = (int) pow(2,rot);

	XRRSetScreenConfig(XDisplay, XConfig, rootWindow, current_Xsize, XRotation, CurrentTime);
	}

}


void AutoRotation::closeEvent ( QCloseEvent * event )
{
     	hide();
        event->ignore();
}


void AutoRotation::iconActivated(QSystemTrayIcon::ActivationReason)
{
    showMaximized ();
}


void AutoRotation::CheckButtonClicked(bool newstate)
{
    	if ( newstate )	CheckButton->setText(OnOffString.at(1));
	else	CheckButton->setText(OnOffString.at(0));
	Settings->setValue("enabled",CheckButton->isChecked());
}


void AutoRotation::DialogButtonClicked(QFlags<OMFlags::Flag> flags)
{
    if ( flags.testFlag(OMFlags::QuitButton) ) qApp->quit();
    if ( flags.testFlag(OMFlags::CloseButton) ) hide();
}

/*!
    \fn AutoRotation::manualRotationSelected(int r)
 */
void AutoRotation::manualRotationSelected(int r)
{
    	rot = r;
	newrot = rot;
	rotateScreen();
	
}
