/***************************************************************************
 *   Copyright (C) 2009 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "idlehandler.h"
#include <QMessageBox>
#include <QIcon>
#include <QPushButton>
#include <QWidget>
#include <QFont>
#include <QDBusReply>
#include "devinterface.h"
#include <omcheckbutton.h>
#include <ombatteryinterface.h>

IdleHandler::IdleHandler()
{

	

	QIcon icon("/usr/share/pixmaps/openmoko-panel-plugin-qt/idlehandler.png");
	setWindowIcon(icon); 
	systrayicon = new QSystemTrayIcon(icon, this);

	connect(systrayicon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	systrayicon->show();

	QuitButton = new QPushButton(QObject::tr("quit"),this);
	QuitButton->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum);
	connect(QuitButton,SIGNAL(clicked()), qApp, SLOT(quit()));

	CloseButton = new QPushButton(QObject::tr("close"),this);
	CloseButton->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum);
	connect(CloseButton,SIGNAL(clicked()), this, SLOT(close()));

	SaveButton = new QPushButton(QObject::tr("save"),this);
	SaveButton->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum);
	connect(SaveButton,SIGNAL(clicked()), this, SLOT(saveSettings()));


	tabWidget[0] = new QWidget(this);
	tabWidget[1] = new QWidget(this);
	tabWidget[2] = new QWidget(this);
	tabView = new QTabWidget(this);
	tabView->addTab(tabWidget[GENTAB],QObject::tr("general"));
	tabView->addTab(tabWidget[APPTAB],QObject::tr("applications"));
	tabView->addTab(tabWidget[BRITAB],QObject::tr("timeout/brightness"));
	
	makeDBUSConnection();


	idlestates << "busy" << "idle" << "idle_dim" << "idle_prelock" << "lock" << "suspend" << "awake";


    	settings = new QSettings("openmoko","idlehandler");
	settings->sync ();

	//Layouts erzeugen
	//Hauptansicht
	MainLayout = new QGridLayout();
	MainLayout->setContentsMargins(0,0,0,0);
	MainLayout->setSpacing(SPACING);

	QuitCloseLayout = new QHBoxLayout();
	QuitCloseLayout-> addWidget( CloseButton );
	QuitCloseLayout-> addWidget( QuitButton );
	MainLayout-> addWidget( tabView, 0, 0 );
	MainLayout-> addWidget( SaveButton,1,0 );
	MainLayout-> addLayout( QuitCloseLayout,2,0 );
	setLayout( MainLayout );

	//Tab General
	TabLayout[GENTAB] = new QVBoxLayout();



	createUSBInterface();
	createGPSInterface();
	createCALLInterface();
	createBTInterface();
	createWiFiInterface();
	
	createAppInterface(tabWidget[APPTAB]);

	

	createStatesettings();

	tabView->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum);
	



	int n;
	//Tab Apps
	TabLayout[APPTAB] = new QVBoxLayout();
	for (n=0; n < MAXAPPS; n++) TabLayout[APPTAB]->addWidget( APP[n] );

	//Tab Statesettings;
	TabLayout[BRITAB] = new QVBoxLayout();
	TabLayout[BRITAB]->addWidget( SettingsTableHead );
	for (n=0; n<=5; n++) TabLayout[BRITAB]->addWidget( StateSettings[n] );

	//layouts den Tabs zuweisen
	for ( n = 0; n < 3; n++) {
		TabLayout[n]->setSpacing(SPACING);
		TabLayout[n]->setContentsMargins(0,SPACING,0,SPACING);
		tabWidget[n]->setLayout( TabLayout[n] );
		}

	

}


IdleHandler::~IdleHandler()
{
//    QDBusConnection::disconnectFromBus("dbus");
}


void IdleHandler::iconActivated(QSystemTrayIcon::ActivationReason)
{
    showMaximized ();
}


void IdleHandler::closeEvent ( QCloseEvent * event )
{
     	hide();
        event->ignore();
}





void IdleHandler::resizeEvent( QResizeEvent *event)
{

	int y = event->size().height() * TABHEIGHT / 100;
	MainLayout->setRowMinimumHeight(0, y);

}


void IdleHandler::IdleStateChanged(QString state)
{

	qDebug() << "receiving Signal " << state; 
	int n;

	bool goback = false;
	if (state == USB->getPreventState()) goback = true ;
	if (state == GPS->getPreventState()) goback = true ;
	if (state == CALL->getPreventState()) goback = true ;
	if (state == BT->getPreventState()) goback = true ;	
	if (state == WiFi->getPreventState()) goback = true ;	

	for (n=0; n < MAXAPPS; n++) {
		if (state == APP[n]->getPreventState()) goback = true ;
	}

	n = idlestates.indexOf( state, 0);

	if (goback) {
		n = n -1;
		idleinterface->call("SetState",idlestates.at(n));
		qDebug() << "Sending Signal " << idlestates.at(n); 

	
		}
	else {
		if (state == "suspend") usageinterface->call("Suspend");
		qDebug() << "Sending Signal " << state;
		
		}

	QDBusMessage reply = displayinterface->call("SetBrightness", StateSettings[n]->brightness());
	qDebug() << "setting brightness: " << StateSettings[n]->brightness();
		

}


void IdleHandler::createUSBInterface()
{
    	USB = new DeviceInterface("USB", tabView->widget(GENTAB), idlestates );
	
	QStringList match; 
	match << "full" << "charging";

	USB->setDBusArguments("org.freesmartphone.odeviced", OMBatteryPath(), "org.freesmartphone.Device.PowerSupply","GetPowerStatus","", match);
	
	USB->loadSettings( settings );

	TabLayout[GENTAB]->addWidget( USB );


}


void IdleHandler::createGPSInterface()
{
	GPS = new DeviceInterface("GPS", tabView->widget(GENTAB), idlestates);
	
	QStringList match; 
	match << "true";

	GPS->setDBusArguments("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage","GetResourceState","GPS", match);
	GPS->loadSettings( settings );

	TabLayout[GENTAB]->addWidget( GPS );

}



void IdleHandler::createCALLInterface()
{
    	CALL = new DeviceInterface("Call", tabView->widget(GENTAB), idlestates);

	QStringList match; 
	match << "gsmhandset";

	CALL->setDBusArguments("org.freesmartphone.odeviced", "/org/freesmartphone/Device/Audio", "org.freesmartphone.Device.Audio","GetScenario","", match);

	CALL->loadSettings( settings );

	TabLayout[GENTAB]->addWidget( CALL );

}


/*!
    \fn IdleHandler::saveSettings()
 */
void IdleHandler::saveSettings()
{
	USB->saveSettings( settings );
	GPS->saveSettings( settings );
	CALL->saveSettings( settings );
	BT->saveSettings( settings );
	WiFi->saveSettings( settings );

	int n;

	for (n=0;n<=3;n++) APP[n]->saveSettings( settings );

	for (n=0; n<=5; n++) StateSettings[n]->saveSettings( settings);

	settings->sync ();   
}



void IdleHandler::createAppInterface( QWidget *tab)
{
	int n;
	for (n = 0; n < MAXAPPS; n++) {
       	APP[n] = new AppInterface(QObject::tr("Application") + QString::number(n,10), tab, idlestates);
	APP[n]->loadSettings( settings );
	}
}


/*!
    \fn IdleHandler::makeDBUSConnection()
 */
void IdleHandler::makeDBUSConnection()
{
  	do {	qDebug() << "Trying to connect to idlenotifier";
		idleinterface = new QDBusInterface("org.freesmartphone.odeviced", "/org/freesmartphone/Device/IdleNotifier/0", "org.freesmartphone.Device.IdleNotifier", QDBusConnection::systemBus(), this);
		if (!idleinterface->isValid ()) { qDebug() << "Failed to connect to idlenotifier"; sleep(5);}
		}
	while (!idleinterface->isValid ());
	qDebug() << "idlenotifier connected";

	connect(idleinterface,SIGNAL( State(QString) ), this, SLOT(IdleStateChanged(QString)));

	do {	qDebug() << "Trying to connect to ousaged";
		usageinterface = new OMUsageInterface( this);
		if (!usageinterface->isValid ()) { qDebug() << "Failed to connect to ousaged"; sleep(5);}
		}
	while (!usageinterface->isValid ());
	qDebug() << "ousaged connected";

	do {	qDebug() << "Trying to connect to display";
		displayinterface = new QDBusInterface("org.freesmartphone.odeviced", "/org/freesmartphone/Device/Display/0", "org.freesmartphone.Device.Display", QDBusConnection::systemBus(), this);
		if (!idleinterface->isValid ()) { qDebug() << "Failed to connect to display"; sleep(5);}
		}
	while (!idleinterface->isValid ());
	qDebug() << "display connected";
}

void IdleHandler::createStatesettings()
{
	QStringList HeadItems;
//	HeadItems << QObject::tr("idlestate") << QObject::tr("time") << QLabel(QObject::tr("bright");
	HeadItems << "idlestate" << "time" << "bright";
	SettingsTableHead = new QTablehead(HeadItems, tabWidget[BRITAB]);

	int n;

	for (n=0; n<=5; n++) {
	StateSettings[n] = new QStateSettings(idlestates.at(n),	tabWidget[BRITAB], idleinterface);
	StateSettings[n]->loadSettings( settings );
	}

	StateSettings[0]->disableTimeout();
	StateSettings[5]->disableBrightness();

	StateSettings[0]->setTimeoutMinimum( 1 );
	StateSettings[0]->setBrightnessMinimum( 50 );

}




/*!
    \fn IdleHandler::createBTInterface()
 */
void IdleHandler::createBTInterface()
{
    	BT = new BTDeviceInterface("Bluetooth", tabView->widget(GENTAB), idlestates);
	BT->loadSettings( settings );

	TabLayout[GENTAB]->addWidget( BT );

}


/*!
    \fn IdleHandler::testRunning()
 */
bool IdleHandler::activate()
{
	showMaximized ();
    	return true;
}

/*!
    \fn IdleHandler::createWiFiInterface()
 */
void IdleHandler::createWiFiInterface()
{
   	WiFi = new DeviceInterface("Wifi", tabView->widget(GENTAB), idlestates);
	
	QStringList match; 
	match << "true";

	WiFi->setDBusArguments("org.freesmartphone.odeviced", "/org/freesmartphone/Device/PowerControl/WiFi", "org.freesmartphone.Device.PowerControl","GetPower","", match);
	WiFi->loadSettings( settings );

	TabLayout[GENTAB]->addWidget( WiFi );


}
