/***************************************************************************
 *   Copyright (C) 2010 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "omapplication.h"
#include <QDebug>

OMApplication::OMApplication( int & argc, char ** argv, QString name )
 : QApplication( argc, argv )
{

	Name = name;
	ServiceName = "org.ompanelpluginqt." + Name;
	ServicePath = "/";

	SessionBus = new QDBusConnection( QDBusConnection::sessionBus() );
/*	if ( !isRegisteredAtSessionBus() )
		registerAtSessionBus();
	else {
		qDebug() << Name + "is already running. stop.";
		quit();
		}
*/	
	loadStyle();


}


OMApplication::~OMApplication()
{
}



void OMApplication::setMainWindow( QWidget *mw)
{
 	MainWindow = mw;  
}


void OMApplication::show()
{
	if ( MainWindow != 0 ) MainWindow->showMaximized();
}


void OMApplication::loadStyle()
{
    	FileList << "/etc/openmoko-panel-plugin-qt/style.conf";

	foreach(QString StyleFile, FileList) {
		QFile File(StyleFile);
		if (File.open(QIODevice::ReadOnly | QIODevice::Text )) {
				QTextStream in(&File);
				Style = in.readAll();
			}

		}
	setStyleSheet(Style);

	qDebug() << Translator.load("/usr/share/openmoko-panel-plugin-qt/translations/" + QLocale::system().name());
        installTranslator(&Translator);
}


void OMApplication::registerAtSessionBus()
{
    	
	if (SessionBus->registerService( ServiceName ) ) {
		if (!SessionBus->registerObject( ServicePath, this, QDBusConnection::ExportAllSlots ) )
			qDebug() << "cannot register Object" << SessionBus->lastError().message();;
	
	} else qDebug() << "cannot register Service:" << SessionBus->lastError().message();
	
}


bool OMApplication::isRunning()
{
	bool res = false;
    	if (SessionBus->isConnected()) { 
		SessionInterface = new QDBusInterface(ServiceName, ServicePath,"",*SessionBus, 0);
		if (SessionInterface->isValid() ) {
			res = true;
			qDebug() << Name + "is already running.";
		}
	}
	return res;
}





void OMApplication::activate()
{
    	if ( SessionInterface->isValid() )
	SessionInterface->call("show");
}


int OMApplication::run()
{
	registerAtSessionBus();
    	return exec();
}
